/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2011-2012 Razor team
 * Authors:
 *   Petr Vanek <petr@scribus.info>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QIcon>
#include "policykitagentgui.h"
#include <unistd.h>

#include <PolkitQt1/Subject>
#include <QTime>
#include <QMessageBox>
#include <QKeyEvent>
#include <QDebug>

namespace NdePolicykit
{


PolicykitAgentGUI::PolicykitAgentGUI(const QString &actionId,
                                     const QString &message,
                                     const QString &iconName,
                                     const PolkitQt1::Details &details,
                                     const PolkitQt1::Identity::List &identities)
   : QDialog(NULL)
{
    setupUi(this);
    Q_UNUSED(actionId);
    Q_UNUSED(details); // it seems too confusing for end user (=me)
    
   // setWindowFlag(Qt::FramelessWindowHint);
    setWindowFlags(Qt::Dialog | Qt::FramelessWindowHint|Qt::WindowStaysOnTopHint);

    
//    QIcon icon = QIcon::fromTheme(iconName);
//    if (icon.isNull())
//        icon = QIcon::fromTheme(QLatin1String("dialo-question"));
//    iconLabel->setPixmap(icon.pixmap(64, 64));

    const uid_t current_uid = getuid();
    int current_user_index = -1;
    for (const PolkitQt1::Identity& identity : identities)
    {
        const int i = identityComboBox->count(); // index of the added item
        identityComboBox->addItem(identity.toString());
        PolkitQt1::UnixUserIdentity const * const u_id = static_cast<const PolkitQt1::UnixUserIdentity *>(&identity);
        if (u_id != nullptr && u_id->uid() == current_uid)
            current_user_index = i;
    }
    if (current_user_index != -1)
        identityComboBox->setCurrentIndex(current_user_index);
    connect(identityComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &PolicykitAgentGUI::onIdentityChanged);


    passwordEdit->setFocus(Qt::OtherFocusReason);

    passwordEdit->installEventFilter(this);



    m_backGround = new fullWindow();
    m_backGround->setAttribute(Qt::WA_DeleteOnClose,true);

    QPalette pal;
    pal.setColor(QPalette::Background, QColor(0, 0, 0, 25));
    m_backGround->setAutoFillBackground(true);
    m_backGround->setPalette(pal);

    m_backGround->show();

    m_backGround->hide();

    setParent(m_backGround);

}

PolicykitAgentGUI::~PolicykitAgentGUI()
{
    m_backGround->deleteLater();
}

void PolicykitAgentGUI::setPrompt(const PolkitQt1::Identity &identity, const QString &text, bool echo)
{
    const int ix = identityComboBox->findText(identity.toString());
    if (ix != -1)
    {
        identityComboBox->setItemData(ix, text, RolePromptText);
        identityComboBox->setItemData(ix, echo, RolePromptEcho);

        if (ix == identityComboBox->currentIndex())
        {
           // promptLabel->setText(text);
            passwordEdit->setEchoMode(echo ? QLineEdit::Normal : QLineEdit::Password);
        }
    }
}

QString PolicykitAgentGUI::identity()
{
    Q_ASSERT(identityComboBox->currentIndex() != -1);
    return identityComboBox->currentText();
}

QString PolicykitAgentGUI::response()
{
    return passwordEdit->text();
}

void PolicykitAgentGUI::showWin()
{
    m_backGround->showFullScreen();
    showFullScreen();
}

void PolicykitAgentGUI::hideWin()
{
    m_backGround->hide();
    hide();
}

void PolicykitAgentGUI::onIdentityChanged(int index)
{
    QVariant text = identityComboBox->itemData(index, RolePromptText);
    QVariant echo = identityComboBox->itemData(index, RolePromptEcho);
    if (text != QVariant{})
       // promptLabel->setText(text.toString());
    if (echo != QVariant{})
        passwordEdit->setEchoMode(echo.toBool() ? QLineEdit::Normal : QLineEdit::Password);
}

} // namespace

void NdePolicykit::PolicykitAgentGUI::on_displayPassword_clicked()
{
    static int flage = 0;
    if(flage == 0){
        passwordEdit->setEchoMode(QLineEdit::Normal);
        displayPassword->setIcon(QIcon(tr(":/src/image/displayPassword.png")));
        flage = 1;
    }else{
        passwordEdit->setEchoMode(QLineEdit::Password);
        displayPassword->setIcon(QIcon(tr(":/src/image/noDisplayPassword.png")));
        flage = 0;
    }
}

void NdePolicykit::PolicykitAgentGUI::on_passwordEdit_textChanged(const QString &arg1)
{
    //修改密码，恢复密码提示状态
    frame->setStyleSheet(QLatin1String("QFrame{border:1px solid #f2f2f2;}"));
    passwordError->setText(tr(""));
}
void NdePolicykit::PolicykitAgentGUI::authorize_signal()
{
    Q_EMIT authorized_inspection_signed();
}

void NdePolicykit::PolicykitAgentGUI::cancle_signal()
{
    Q_EMIT authorized_cancle_signed();
}

bool NdePolicykit::PolicykitAgentGUI::eventFilter(QObject *target, QEvent *event)
{
    if(target == passwordEdit)		
        {
            if(event->type() == QEvent::KeyPress)
            {
                 QKeyEvent *k = static_cast<QKeyEvent *>(event);

                 if(k->key() == Qt::Key_Return)
                 {

                     if(passwordEdit->text().isEmpty())
                     {
                         frame->setStyleSheet(QLatin1String("QFrame{border:1px solid #ff3535;}"));
                         passwordError->setText(tr("The password cannot be empty"));
                     }else{
                         authorize_signal();		
                     }
                     return true;
                 }
            }
        }
    return QWidget::eventFilter(target,event);
}





void NdePolicykit::PolicykitAgentGUI::on_OK_clicked()
{
    if(passwordEdit->text().isEmpty())
    {
        frame->setStyleSheet(QLatin1String("QFrame{border:1px solid #ff3535;}"));
        passwordError->setText(tr("The password cannot be empty"));
    }else{
        authorize_signal();
    }



}

void NdePolicykit::PolicykitAgentGUI::on_Cancel_clicked()
{
    cancle_signal();

}

void NdePolicykit::PolicykitAgentGUI::on_closeButton_released()
{
     cancle_signal();

}
